//Copyright 2006 Emily Shen, licensed under the GPLv2 as described in COPYING

#ifndef FSSIG_H
#define FSSIG_H

#include <pbc.h>

struct fs_sys_param_s {
  pairing_ptr pairing;
  int sig_length;
};
typedef struct fs_sys_param_s fs_sys_param_t[1];
typedef struct fs_sys_param_s *fs_sys_param_ptr;

struct fs_public_key_s {
  fs_sys_param_ptr param;
  int T;
  int t;
  element_t g1, g2, e_g1_g2_v, e_g1_g2_w, u_prime, v_prime;
  element_t *U, *V;
};
typedef struct fs_public_key_s fs_public_key_t[1];
typedef struct fs_public_key_s *fs_public_key_ptr;

struct sk_node_s {
  element_t *sk_vector;
  struct sk_node_s *left;  
  struct sk_node_s *right;
  struct sk_node_s *parent;
  int depth;
};
typedef struct sk_node_s sk_node_t[1];
typedef struct sk_node_s *sk_node_ptr;

struct fs_private_key_s {
  fs_sys_param_ptr param;
  int T;
  int t;
  int time;
  sk_node_ptr root;
};
typedef struct fs_private_key_s fs_private_key_t[1];
typedef struct fs_private_key_s *fs_private_key_ptr;


struct fs_dec_key_s {
  element_t dk;
};
typedef struct fs_dec_key_s fs_dec_key_t[1];
typedef struct fs_dec_key_s *fs_dec_key_ptr;

int fs_verify(unsigned char *sig, unsigned char *msg, 
	      unsigned int msg_len_bits, int time, fs_public_key_t pk);
void fs_sign(unsigned char *sig, unsigned char *msg, unsigned int msg_len_bits,
	     fs_public_key_t pk, fs_private_key_t sk, fs_dec_key_t dk);
void fs_update(fs_private_key_t sk, fs_public_key_t pk);
void fs_gen_sys_param(fs_sys_param_t param, pairing_t pairing);
void fs_gen(fs_public_key_t pk, fs_private_key_t sk, fs_dec_key_t dk, fs_sys_param_t param, unsigned int msg_len_bits, unsigned int num_intervals);
int fs_check_key(fs_private_key_t sk, fs_public_key_t pk);

#endif //FSSIG_H
