#include "fssig.h"
#include "get_time.h"

int main(void)
{
  pairing_t pairing;
  fs_sys_param_t param;
  fs_public_key_t pk;
  fs_private_key_t sk;
  fs_dec_key_t dk;
  unsigned char *sig;
  int i, n, j;
  double t0, t1, ttotal, stotal, vtotal, utotal, ctotal;

  printf("reading pairing from stdin...\n");
  pairing_init_inp_str(pairing, stdin);
  
  n = 10;
  ttotal = stotal = vtotal = utotal = ctotal = 0.0;
  for (j = 0; j < n; j++) {
    printf("generating system parameters/key pair #%d...\n", j);
    t0 = get_time();
    fs_gen_sys_param(param, pairing);
    fs_gen(pk, sk, dk, param, 160, 128);
    t1 = get_time();
    ttotal += t1 - t0;

    sig = (unsigned char *) malloc(param->sig_length);
    for (i = 0; i < 128; i++) {
      printf("checking key #%d...\n", i);
      t0 = get_time();
      fs_check_key(sk, pk);
      t1 = get_time();
      ctotal += t1 - t0;
      printf("signing #%d...\n", i);
      t0 = get_time();
      fs_sign(sig, (unsigned char *) "hellowor", 160, pk, sk, dk);
      t1 = get_time();
      stotal += t1 - t0;
      printf("verifying #%d...\n", i);
      t0 = get_time();
      fs_verify(sig, (unsigned char *) "hellowor", 160, i, pk);
      t1 = get_time();
      vtotal += t1 - t0;
      printf("updating #%d...\n", i);
      t0 = get_time();
      fs_update(sk, pk);
      t1 = get_time();
      utotal += t1 - t0;
      }
    }
  printf("average keygen time = %f\n", ttotal / n);
  printf("average check time = %f\n", ctotal / (n*128));
  printf("average sign time = %f\n", stotal / (n*128));
  printf("average verify time = %f\n", vtotal / (n*128));
  printf("average update time = %f\n", utotal / (n*128));
  return 0;
}
